
import logging

from collections import defaultdict

try: # python-requests sometimes use json, sometimes use simplejson
    from simplejson import JSONDecodeError
    JSON_ERROR = JSONDecodeError
except ImportError:
    JSON_ERROR = (TypeError, ValueError)

from django import forms
from django.contrib.auth.forms import UserChangeForm as AuthUserChangeForm, UserCreationForm as AuthUserCreationForm
from django.contrib.auth.forms import ReadOnlyPasswordHashField
from django.contrib.auth import get_user_model
from django.utils.translation import ugettext_lazy as _


from authentic2.forms import EmailChangeForm

from . import widgets


logger = logging.getLogger()

# these two forms are borrowed and adapted from django.contrib.auth.forms

class UserChangeForm(forms.ModelForm):
    email = forms.EmailField(label=_("Email"))
    password = ReadOnlyPasswordHashField(label=_("Password"),
        help_text=_("Raw passwords are not stored, so there is no way to see "
                    "this user's password, but you can change the password "
                    "using <a href=\"password/\">this form</a>."))

    class Meta:
        model = get_user_model()

    def __init__(self, *args, **kwargs):
        super(UserChangeForm, self).__init__(*args, **kwargs)
        f = self.fields.get('user_permissions', None)
        if f is not None:
            f.queryset = f.queryset.select_related('content_type')

    def clean_password(self):
        # Regardless of what the user provides, return the initial value.
        # This is done here, rather than on the field, because the
        # field does not have access to the initial value
        return self.initial["password"]

    def clean_email(self):
        # Since User.email is unique, this check is redundant,
        # but it sets a nicer error message than the ORM. See #13147.
        email = self.cleaned_data["email"]
        if email == self.initial['email']:
            return email
        User = get_user_model()
        try:
            User._default_manager.get(email=email)
        except User.DoesNotExist:
            return email
        raise forms.ValidationError(_("Email already taken"))


class UserCreationForm(forms.ModelForm):
    error_messages = {
        'duplicate_email': _("A user with that email already exists."),
        'password_mismatch': _("The two password fields didn't match."),
    }
    email = forms.EmailField(label=_("Email"))
    password1 = forms.CharField(label=_("Password"),
        widget=forms.PasswordInput)
    password2 = forms.CharField(label=_("Password confirmation"),
        widget=forms.PasswordInput,
        help_text=_("Enter the same password as above, for verification."))

    class Meta:
        model = get_user_model()
        fields = ("email",)

    def clean_email(self):
        # Since User.email is unique, this check is redundant,
        # but it sets a nicer error message than the ORM. See #13147.
        email = self.cleaned_data["email"]
        User = get_user_model()
        try:
            User._default_manager.get(email=email)
        except User.DoesNotExist:
            return email
        raise forms.ValidationError(self.error_messages['duplicate_email'])

    def clean_password2(self):
        password1 = self.cleaned_data.get("password1")
        password2 = self.cleaned_data.get("password2")
        if password1 and password2 and password1 != password2:
            raise forms.ValidationError(
                self.error_messages['password_mismatch'])
        return password2

    def save(self, commit=True):
        user = super(UserCreationForm, self).save(commit=False)
        user.set_password(self.cleaned_data["password1"])
        if commit:
            user.save()
        return user

class MeyzieuNewslettersForm(forms.Form):
    fil_actu = forms.ChoiceField(label='Fil actu', choices=(),
            widget=forms.RadioSelect, required=False)
    cap_meyzieu = forms.ChoiceField(label='Cap Meyzieu', choices=(),
            widget=forms.RadioSelect, required=False)
    autres_publications = forms.ChoiceField(label='Autres publications', choices=(),
            widget=forms.RadioSelect, required=False)
    push_mail = forms.MultipleChoiceField(label='Push mail', choices=(),
            widget=widgets.CheckboxMultipleSelect, required=False)
    newsletter = forms.MultipleChoiceField(label='Newsletters', choices=(),
            widget=widgets.CheckboxMultipleSelect, required=False)


    def __init__(self, *args, **kwargs):
        self.request = kwargs.pop('request')
        self.user = self.request.user
        self.instance = kwargs.pop('instance')
        self.none_line = False
        super(MeyzieuNewslettersForm, self).__init__(*args, **kwargs)
        if self.data:
            self.data2 = {}
            for key in self.data:
                self.data2[key] = self.data.getlist(key)
        try:
            self.newsletters = sorted(self.instance.get_newsletters(), key=lambda x: x['name'])
        except JSON_ERROR:
            logger.warning('failure to retrieve newsletters list: JSONDecodeError')
            self.newsletters = []
        try:
            self.subscriptions = self.instance.get_subscriptions(
                    email=self.user.email)
        except JSON_ERROR:
            logger.warning('failure to retrieve current subscriptions: JSONDecodeError')
            self.subscriptions = []
        choices = {
                'fil_actu': [],
                'cap_meyzieu': [],
                'autres_publications': [],
                'push_mail': [],
                'newsletter': [],
        }
        initials = {
                'fil_actu': '',
                'cap_meyzieu': '',
                'autres_publications': '',
                'push_mail': [],
                'newsletter': [],
        }

        for key, value in initials.iteritems():
            if not isinstance(value, list):
                choices[key] = [('', u'Aucun abonnement')]
        if self.instance.get_kinds_restriction():
            for kind in self.fields.keys():
                if kind not in self.instance.get_kinds_restriction():
                    del self.fields[kind]
                    del choices[kind]
                    del initials[kind]

        for newsletter in self.newsletters:
            field_id = self.to_field_id(newsletter)
            newsletter['field_id'] = field_id
            for t in newsletter['type']:
                choices[t].append((field_id, newsletter['name']))
                if initials[t] == '':
                    self.none_line = True
        for newsletter in self.subscriptions:
            field_id = self.to_field_id(newsletter)
            for t in newsletter['type']:
                if isinstance(initials[t], list):
                    initials[t].append(field_id)
                else:
                    initials[t] = field_id
        for t in choices:
            self.fields[t].choices = choices[t]
        for t in initials:
            self.fields[t].initial = initials[t]

    def to_field_id(self, newsletter):
        return 'newsletter_%s' % newsletter['newsletter_id']

    def from_field_id(self, field_id):
        x, newsletter_id = field_id.split('_')
        return int(newsletter_id)

    def save(self):
        cleaned_data = self.cleaned_data
        subscriptions = defaultdict(lambda: [])
        for t in cleaned_data:
            if isinstance(cleaned_data[t], (list, tuple)):
                for field_id in cleaned_data[t]:
                    newsletter_id = self.from_field_id(field_id)
                    subscriptions[newsletter_id].append(t)
            else:
                if cleaned_data[t]:
                    newsletter_id = self.from_field_id(cleaned_data[t])
                    subscriptions[newsletter_id].append(t)
        subscriptions = [ dict(newsletter_id=i, type=types) for i, types in subscriptions.iteritems() ]
        try:
            self.instance.set_subscriptions(self.user.email, subscriptions)
        except JSON_ERROR:
            logger.warning('failure to set new subscriptions: JSONDecodeError')

class EmailChangeForm(EmailChangeForm):

    def clean_email(self):
        # Since User.email is unique, this check is redundant,
        # but it sets a nicer error message than the ORM. See #13147.
        email = self.cleaned_data["email"]
        User = get_user_model()
        try:
            User._default_manager.get(email=email)
        except User.DoesNotExist:
            return email
        raise forms.ValidationError(_("Email already taken."))

