# -*- coding: utf-8 -*-
from collections import defaultdict
import logging


import requests
import warnings

from django.core.exceptions import ImproperlyConfigured
from django.core.mail import send_mail
from django.db import models
from django.contrib.auth.models import AbstractBaseUser, PermissionsMixin, BaseUserManager, SiteProfileNotAvailable
from django.core.validators import RegexValidator, MinLengthValidator, \
    MaxLengthValidator
from django.utils.http import urlquote
from django.utils.translation import ugettext_lazy as _, pgettext_lazy
from django.conf import settings
from django.utils import timezone

logger = logging.getLogger(__name__)

class UserManager(BaseUserManager):
    def create_user(self, email, password=None, **extra_fields):
        """
        Creates and saves a User with the given email and password.
        """
        now = timezone.now()
        if not email:
            raise ValueError('The given email must be set')
        email = UserManager.normalize_email(email)
        user = self.model(email=email,
                          is_staff=False, is_active=True, is_superuser=False,
                          last_login=now, date_joined=now, **extra_fields)
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_superuser(self, email, password, **extra_fields):
        u = self.create_user(email, password, **extra_fields)
        u.is_staff = True
        u.is_active = True
        u.is_superuser = True
        u.save(using=self._db)
        return u

class User(AbstractBaseUser, PermissionsMixin):

    objects = UserManager()


    is_staff = models.BooleanField(_('staff status'), default=False,
        help_text=_('Designates whether the user can log into this admin '
                    'site.'))
    is_active = models.BooleanField(_('active'), default=True,
        help_text=_('Designates whether this user should be treated as '
                    'active. Unselect this instead of deleting accounts.'))
    date_joined = models.DateTimeField(_('date joined'), default=timezone.now)
    backend = models.CharField(max_length=64, blank=True)
    backend_id = models.CharField(max_length=256, blank=True)

    title = models.CharField(
            verbose_name=unicode(pgettext_lazy('person title', 'title')),
            max_length=16,
            blank=True,
            choices=tuple((x, x) for x in (_('Mrs'), _('Mr'),)))

    first_name = models.CharField(_('first name'), max_length=30, blank=True)
    last_name = models.CharField(_('last name'), max_length=30, blank=True)
    email = models.EmailField(_('e-mail address'), max_length=128, blank=True,
            unique=True)
    city = models.CharField(_('city'), max_length=64, blank=True)
    address = models.CharField(_('address'), max_length=128, blank=True)
    postal_code = models.CharField(_('postal code'), max_length=5, blank=True,
            validators=[
                RegexValidator(r'^[0-9]*$',
                    _('Postal code must be five numbers'),
                    _('Invalid postal code')),
                MinLengthValidator(5),
                MaxLengthValidator(5)])
    fax = models.CharField(verbose_name=_("fax"),
            max_length=16, blank=True)
    phone = models.CharField(verbose_name=_("phone"),
            max_length=16, blank=True)
    mobile = models.CharField(verbose_name=_("mobile"),
            max_length=16, blank=True)
    old_username = models.CharField(max_length=256,
            blank=True)


    @property
    def username(self):
        return '%s' % self.email

    REGISTER_FIELDS = ('email', 'title', 'last_name', 'first_name',
            'mobile', 'phone', 'address', 'postal_code',  'city')
    REQUIRED_FIELDS = ['first_name', 'last_name', 'mobile']
    USERNAME_FIELD = 'email'
    USER_PROFILE = ('title', 'email', 'last_name', 'first_name',
            'address', 'postal_code',  'city', 'fax', 'mobile', 'phone', 'roles')

    class Meta:
        verbose_name = _('user')
        verbose_name_plural = _('users')

    def get_absolute_url(self):
        return "/users/%s/" % urlquote(self.email)

    def get_full_name(self):
        """
        Returns the first_name plus the last_name, with a space in between.
        """
        full_name = '%s %s' % (self.first_name, self.last_name)
        return full_name.strip()

    def get_short_name(self):
        "Returns the short name for the user."
        return self.email

    def email_user(self, subject, message, from_email=None):
        """
        Sends an email to this User.
        """
        send_mail(subject, message, from_email, [self.email])

    def get_profile(self):
        """
        Returns site-specific profile for this user. Raises
        SiteProfileNotAvailable if this site does not allow profiles.
        """
        warnings.warn("The use of AUTH_PROFILE_MODULE to define user profiles has been deprecated.",
            DeprecationWarning, stacklevel=2)
        if not hasattr(self, '_profile_cache'):
            from django.conf import settings
            if not getattr(settings, 'AUTH_PROFILE_MODULE', False):
                raise SiteProfileNotAvailable(
                    'You need to set AUTH_PROFILE_MODULE in your project '
                    'settings')
            try:
                app_label, model_name = settings.AUTH_PROFILE_MODULE.split('.')
            except ValueError:
                raise SiteProfileNotAvailable(
                    'app_label and model_name should be separated by a dot in '
                    'the AUTH_PROFILE_MODULE setting')
            try:
                model = models.get_model(app_label, model_name)
                if model is None:
                    raise SiteProfileNotAvailable(
                        'Unable to load the profile model, check '
                        'AUTH_PROFILE_MODULE in your project settings')
                self._profile_cache = model._default_manager.using(
                                   self._state.db).get(user__id__exact=self.id)
                self._profile_cache.user = self
            except (ImportError, ImproperlyConfigured):
                raise SiteProfileNotAvailable
        return self._profile_cache

    def get_backend(self):
        return load_backend(self.backend)

    def get_roles(self):
        return [group.name for group in self.groups.all()]
    roles = property(get_roles)

from authentic2.attribute_aggregator.core import ATTRIBUTE_MAPPING

ATTRIBUTE_MAPPING['l']['profile_field_name'] = 'city'
ATTRIBUTE_MAPPING['personalTitle']['profile_field_name'] = 'title'
ATTRIBUTE_MAPPING['street']['profile_field_name'] = 'address'
ATTRIBUTE_MAPPING['postalCode']['profile_field_name'] = 'postal_code'
ATTRIBUTE_MAPPING['mobile']['profile_field_name'] = 'mobile'

from cms.models import CMSPlugin

class MeyzieuNewslettersPlugin(CMSPlugin):
    url = models.URLField(verbose_name=_('URL du service de newsletter'))
    apikey = models.CharField(max_length=32, verbose_name=_('API key'))
    newsletters_restriction = models.CharField(max_length=1024,
            verbose_name=_('Newsletters to show'),
            help_text=_('list of comma separated values'), default='', blank=True)
    kinds_restriction = models.CharField(max_length=1024,
            verbose_name=('Kinds to show'),
            help_text=_('list of comma separated values'), default='', blank=True)

    subscriptions = []

    __newsletters_restriction = []
    __kinds_restriction = []

    def __init__(self, *args, **kwargs):
        super(MeyzieuNewslettersPlugin, self).__init__(*args, **kwargs)

    def get_newsletters_restriction(self):
        if not self.__newsletters_restriction:
            self.__newsletters_restriction = filter(None, map(unicode.strip, self.newsletters_restriction.split(',')))
        return self.__newsletters_restriction

    def get_kinds_restriction(self):
        if not self.__kinds_restriction:
            self.__kinds_restriction = filter(None, map(unicode.strip, self.kinds_restriction.split(',')))
        return self.__kinds_restriction

    def get_newsletters(self):
        # http://recette.meyzieu.fr/services/?method=&apikey=9057033f8bdcea65e0493b0a04e6a0cf
        try:
            params = {
                'method': 'meyzieu.connection.getListRubrique',
                'apikey': self.apikey
            }
            response = requests.get(self.url, params=params)
        except (requests.HTTPError, requests.ConnectionError), e:
            logger.error('error when getting newsletters  from %r(%r): %r',
                    self.url, params, e)
            return []
        logger.debug('call to ws %s', response.request.url)
        logger.debug('response %s', response.content)
        newsletters = {}
        for kind, value in response.json().iteritems():
            if self.get_kinds_restriction() and kind not in self.get_kinds_restriction():
                continue
            for item in value:
                if self.get_newsletters_restriction() and item['id_rubrique'] not in self.get_newsletters_restriction():
                    continue
                newsletter = newsletters.setdefault(item['id_rubrique'], {})
                newsletter['newsletter_id'] = item['id_rubrique']
                newsletter['name'] = item['titre']
                kinds = newsletter.setdefault('type', [])
                kinds.append(kind)
        return newsletters.values()

    def set_subscriptions(self, email, subscriptions):
        logger.debug('new_subscriptions %s', subscriptions)
        currents = defaultdict(lambda: {'type': []})
        current_subscriptions = self.get_subscriptions(email)
        logger.debug('current_subscriptions %s' % current_subscriptions)
        for current in current_subscriptions:
            if self.get_newsletters_restriction() and current['newsletter_id'] not in self.get_newsletters_restriction():
                continue
            currents[current['newsletter_id']] = current
        news = defaultdict(lambda: {'type': []})
        for new in subscriptions:
            news[unicode(new['newsletter_id'])] = new
        add = defaultdict(lambda:[])
        remove = defaultdict(lambda:[])
        for key in news:
            to_add = set(news[key]['type']) - set(currents[key]['type'])
            for kind in to_add:
                if self.get_kinds_restriction() and kind not in self.get_kinds_restriction():
                    continue
                add[kind].append(key)
        for key in currents:
            to_remove = set(currents[key]['type']) - set(news[key]['type'])
            for kind in to_remove:
                if self.get_kinds_restriction() and kind not in self.get_kinds_restriction():
                    continue
                remove[kind].append(key)
        logger.debug('add %s', add)
        logger.debug('remove %s', remove)
        base = {
            'method': 'meyzieu.connection.addAbonnement',
            'apikey': self.apikey,
            'auteur[email]': email,
        }
        for kind, value in remove.iteritems():
            params = base.copy()
            params.update({
                'type': kind,
                'desabonnement': 1,
            })
            for i, rubrique_id in enumerate(value):
                params['id_abonnement[%d]' % i] = rubrique_id
            response = requests.get(self.url, params=params)
            logger.debug('call to ws %s', response.request.url)
            logger.debug('response %s', response.content)
        for kind, value in add.iteritems():
            params = base.copy()
            params.update({
                'type': kind,
                'desabonnement': 0,
            })
            for i, rubrique_id in enumerate(value):
                params['id_abonnement[%d]' % i] = rubrique_id
            response = requests.get(self.url, params=params)
            logger.debug('call to ws %s', response.request.url)
            logger.debug('response %s', response.content)


    def get_subscriptions(self, email):
        # http://recette.meyzieu.fr/services/?method=&apikey=9057033f8bdcea65e0493b0a04e6a0cf&auteur+%5Bemail%5D=s.besson%40acti.fr
        try:
            params = {
                'method': 'meyzieu.connection.getListAbonnement',
                'apikey': self.apikey,
                'auteur[email]': email,
            }
            response = requests.get(self.url, params=params)
        except (requests.HTTPError, requests.ConnectionError), e:
            logger.error('error when getting subscriptions from %r(%r): %r',
                    self.url, params, e)
            return []
        logger.debug('call to ws %s', response.request.url)
        logger.debug('response %s', response.content)

        if 'abonnement' in response.json():
            newsletters = {}
            for kind, value in response.json()['abonnement'].iteritems():
                if self.get_kinds_restriction() and kind not in self.get_kinds_restriction():
                    continue
                for item in value:
                    if self.get_newsletters_restriction() and item['id_abonnement'] not in self.get_newsletters_restriction():
                        continue
                    newsletter = newsletters.setdefault(item['id_abonnement'], {})
                    newsletter['newsletter_id'] = item['id_abonnement']
                    kinds = newsletter.setdefault('type', [])
                    kinds.append(kind)
            return newsletters.values()
        else:
            return []

    def __unicode__(self):
        return u'meyzieu newsletters plugin %s' % self.url
